from setuptools import setup

setup(
    name='ruko',
    version='0.1.1',
    license='MIT',
    description='A dead simple high performance JSON database',
    author='Matthew Scholefield',
    author_email='matthew331199@gmail.com',
    url='https://github.com/rukodb/ruko',
    packages=['ruko'],
    install_requires=[],
    keywords='ruko database in-memory',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',

        'Topic :: Database :: Database Engines/Servers',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ]
)
